# AUTOGENERATED! DO NOT EDIT! File to edit: ../../../nbs/15_l2r.data.stat.ipynb.

# %% auto 0
__all__ = []

# %% ../../../nbs/15_l2r.data.stat.ipynb 2
# from fastcore.basics import *
# from fastai.torch_core import *
# from fastai.data.core import *
# from fastai.data.transforms import *
# from fastai.text.core import *
# from fastai.text.data import *
# from xcube.imports import *
# from xcube.torch_imports import *
